package com.trust.ywx.customer.common.model.manager.request;

import com.trust.ywx.customer.common.enu.CommonStatusEnum;
import com.trust.ywx.customer.common.enu.DoctorProductTypeEnum;
import com.trust.ywx.customer.common.enu.FirmProperEnum;
import lombok.Data;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <pre></pre>
 *
 * @文件名称： FirmPropertyModel.java
 * @包 路  径： com.trust.ywx.customer.common.model
 * @Copyright：北京数字医信责任有限公司 (C) 2022 *
 * @Description:
 * @Version: V1.0
 * @Author： hanzhenchao
 * @Date：2022/6/16 16:12
 * @Modify：
 */
@Data
public class FirmPropParam implements Serializable {
    private static final long serialVersionUID = 5143239003740244348L;
    /**
     * 计费模式
     * 0-规则计费 1-异常计费
     * see com.trust.ywx.customer.common.enums.ChargingModeEnum
     */
    private Integer chargingMode;

    /**
     * 是否需要实施支持
     * 0-否；1-是
     */
    private Integer techSupport;
    private String techSupportInfo;
    /**
     * 医师产品形态
     * SDK、APP
     */
    private List<String> doctorProductType;
    /**
     * 公众产品形态
     * 0-SDK；1-H5
     * see com.trust.ywx.customer.common.enums.PublicProductTypeEnum
     */
    private Integer publicProductType;

    /**
     * 客户网络类型
     * 1-实体医院；2-互联网医院
     * see com.trust.ywx.customer.common.enums.NetworkTypeEnum
     */
    private Integer networkType;

    /**
     * 内控平台名称
     */
    private String hiscaName;

    /**
     * 内控平台logo图片
     */
    private String hiscaLogo;

    /**
     * 内控平台宣传图片
     */
    private String hiscaPublicityPic;

    /**
     * 内控平台回调地址
     */
    private String hiscaCallUrl;

    /**
     * 通信平台版本 --idm不需要维护此版本
     */
    @Deprecated
    private String gatewayVersion;
    /**
     * 内控监控地址
     */
    private String hiscaMonitorUrl;
    /**
     * 通信平台监控地址
     */
    private String gatewayMonitorUrl;

    /**
     * 客户类型
     * 0-普通 1-冠新 2-微医 3-阿里客户
     * see com.trust.ywx.customer.common.enums.FirmModeEnum
     */
    private Integer firmMode;
//
//    /**
//     * 证书算法 RSA、SM2
//     */
//    private String certAlg;

    /**
     * 客户白名单
     * 0-否；1-是
     * see com.trust.ywx.customer.common.enums.CommonStatusEnum
     */
    private Integer isFirmWhite;

//    /**
//     * ca渠道类型
//     * see com.trust.ywx.customer.common.enums.CaChannelEnum
//     * fixme 目前存放与firm_info表的ca_channel。以后需要迁移到firm_info_extends表
//     */
//    private Integer caChannel;

    /**
     * 是否允许医生挂靠
     * 0-否；1-是
     * see com.trust.ywx.customer.common.enums.CommonStatusEnum
     */
    private Integer allowUserBind;


    /**
     * 是否在挂靠医院列表里展示
     * see com.trust.ywx.customer.common.enums.CommonStatusEnum
     */
    private Integer showFirmInBindList;

    /**
     * 审核完成发送短信
     * see com.trust.ywx.customer.common.enums.CommonStatusEnum
     */
    private Integer smsSend;
    private Integer smsCustomize;
    private String smsCustomizeContent;

    //-----------------医生签名增强-------------
    /**
     * 拒签
     */
    private Integer doctorSignReject;
    /**
     * 作废
     */
    private Integer doctorSignInvalid;
    /**
     * 人脸识别
     */
    private Integer doctorSignFaceDetect;
    //-----------------医生签名增强 end-------------

    /**
     * 公众号签名增强
     * see com.trust.ywx.customer.common.enums.CommonStatusEnum
     */
    private Integer publicSignPhoto;

    /**
     * 个人图章样式
     * see  //stamp-默认；stamp1-签章样式1
     */
    private Integer userStampType;

//------------------个人图章模式-----------------
    /**
     * 自动设置图章
     * fixme 目前存放与firm_info表的stamp_type。以后需要迁移到firm_info_extends表
     */
    private Integer userStampAuto;
    /**
     * 图章压缩
     */
    private Integer userStampCompress;

    /**
     * 图章审核
     */
    private Integer userStampIsCheck;

    /**
     * app设置图章
     */
    private Integer userStampIsAppset;

    //------------------个人图章模式 end-----------------;
    //------------------企业图章模式-----------------;
    /**
     * 厂商图章类型 0 圆章 1 方章
     * see com.trust.ywx.customer.common.enums.FirmStampEnum
     */
    private Integer firmStampType;
    /**
     * 厂商图章时间戳 0否 1是
     */
    private Integer firmStampTS;
    /**
     * 子厂商图章时间戳 0否 1是
     */
    private Integer firmStampDepartTS;

    /**
     * 个人PDF增加水印
     * 0-否；1-是
     * see com.trust.ywx.customer.common.enums.CommonStatusEnum
     */
    private Integer personPdfWater;

    /**
     * 附件pdf增加水印
     */
    private Integer appendPdfWater;

    /**
     * 自动签增强功能 -用户无感知
     */
    private Integer selfSignInsensitive;

    public Map<String,String> convertToMap(){
        Map<String,String> propMap = new HashMap<>(32);
        propMap.put(FirmProperEnum.CHARGING_MODE.getProperName(), String.valueOf(this.chargingMode));
        propMap.put(FirmProperEnum.TECHPERSON_SUPPORT.getProperName(), String.valueOf(this.techSupport));
        propMap.put(FirmProperEnum.TECHPERSON_SUPPORT_INFO.getProperName(), this.techSupportInfo);
        propMap.put(FirmProperEnum.PUBLIC_PRODUCT_TYPE.getProperName(), String.valueOf(this.publicProductType));
        if(null != this.getDoctorProductType() && this.getDoctorProductType().size()>0){
            if(this.getDoctorProductType().contains(DoctorProductTypeEnum.APP.getName())){
                propMap.put(FirmProperEnum.DOCTOR_PRODUCT_APP.getProperName(), CommonStatusEnum.YES.getCode()+"");
            }else{
                propMap.put(FirmProperEnum.DOCTOR_PRODUCT_APP.getProperName(), CommonStatusEnum.NO.getCode()+"");
            }
            if(this.getDoctorProductType().contains(DoctorProductTypeEnum.SDK.getName())){
                propMap.put(FirmProperEnum.DOCTOR_PRODUCT_SDK.getProperName(), CommonStatusEnum.YES.getCode()+"");
            }else{
                propMap.put(FirmProperEnum.DOCTOR_PRODUCT_SDK.getProperName(), CommonStatusEnum.NO.getCode()+"");

            }
        }
        propMap.put(FirmProperEnum.NETWORK_TYPE.getProperName(), String.valueOf(this.networkType));
        propMap.put(FirmProperEnum.GATEWAY_VERSION.getProperName(), String.valueOf(this.gatewayVersion));
        propMap.put(FirmProperEnum.HISCA_NAME.getProperName(), this.hiscaName);
        propMap.put(FirmProperEnum.HISCA_LOGO.getProperName(), this.hiscaLogo);
        propMap.put(FirmProperEnum.HISCA_PUBLICITY.getProperName(), this.hiscaPublicityPic);
        propMap.put(FirmProperEnum.HISCA_MONITOR_URL.getProperName(), this.hiscaMonitorUrl);
        propMap.put(FirmProperEnum.GATEWAY_MONITOR_URL.getProperName(), this.gatewayMonitorUrl);
        propMap.put(FirmProperEnum.HISCA_CALL_URL.getProperName(), this.hiscaCallUrl);
        propMap.put(FirmProperEnum.FIRM_MODE.getProperName(), String.valueOf(this.firmMode));
//        propMap.put(FirmProperEnum.CERT_ALG.getProperName(), this.certAlg);
//        propMap.put(FirmProperEnum.CA_CHANNEL.getProperName(), String.valueOf(this.caChannel));
        propMap.put(FirmProperEnum.FIRM_ALLOW_USER_BIND.getProperName(), String.valueOf(this.allowUserBind));
        propMap.put(FirmProperEnum.FIRM_SHOW_INBINDLIST.getProperName(), String.valueOf(this.showFirmInBindList));
        propMap.put(FirmProperEnum.SMS_SEND.getProperName(), String.valueOf(this.smsSend));
        propMap.put(FirmProperEnum.SMS_CUSTOMIZE.getProperName(), String.valueOf(this.smsCustomize));
        propMap.put(FirmProperEnum.SMS_CUSTOMIZE_CONTENT.getProperName(), this.smsCustomizeContent);
        propMap.put(FirmProperEnum.REJECT_SIGN.getProperName(), String.valueOf(this.doctorSignReject));
        propMap.put(FirmProperEnum.INVALID_SIGN.getProperName(), String.valueOf(this.doctorSignInvalid));
        propMap.put(FirmProperEnum.FACE_DETECT.getProperName(), String.valueOf(this.doctorSignFaceDetect));
        propMap.put(FirmProperEnum.PUBLIC_SIGN_PHOTO.getProperName(), String.valueOf(this.publicSignPhoto));
        propMap.put(FirmProperEnum.USER_STAMP_TYPE.getProperName(), String.valueOf(this.userStampType));
        propMap.put(FirmProperEnum.STAMP_IS_CHECK.getProperName(), String.valueOf(this.userStampIsCheck));
        propMap.put(FirmProperEnum.STAMP_IS_APPCLOSE.getProperName(), String.valueOf(this.userStampIsAppset));
        propMap.put(FirmProperEnum.USER_STAMP_COMPRESS.getProperName(), String.valueOf(this.userStampCompress));
        propMap.put(FirmProperEnum.STAMP_AUTO_SET.getProperName(), String.valueOf(this.userStampAuto));
        propMap.put(FirmProperEnum.FIRM_STAMP_TYPE.getProperName(), String.valueOf(this.firmStampType));
        propMap.put(FirmProperEnum.FIRM_STAMP_TS.getProperName(), String.valueOf(this.firmStampTS));
        propMap.put(FirmProperEnum.DEPART_STAMP_TS.getProperName(), String.valueOf(this.firmStampDepartTS));
        propMap.put(FirmProperEnum.FIRM_PERSONPDF_WATER.getProperName(), String.valueOf(this.personPdfWater));
        propMap.put(FirmProperEnum.FIRM_APPENDPDF_WATER.getProperName(), String.valueOf(this.appendPdfWater));
        propMap.put(FirmProperEnum.SELF_SIGN_BY_FIRM.getProperName(), String.valueOf(this.selfSignInsensitive));

        return propMap;
    }
}
