package com.trust.ywx.customer.common.model.manager.request;

import lombok.Data;

import javax.validation.constraints.Email;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Pattern;
import javax.validation.constraints.Size;
import java.io.Serializable;

@Data
public class TempFirmParam implements Serializable {

    private static final long serialVersionUID = 4072319794402378694L;
    /**
     * 客户ID
     */
    private String clientId;
    /**
     * 客户名称
     */
    @NotBlank(message = "客户名称不能为空")
    @Size(max = 29, message = "客户名称最多29个字符")
    private String firmName;

    /**
     * 客户联系人
     */
    @NotBlank(message = "客户联系人不能为空")
    @Size(max = 10, message = "客户联系人最多10个字符")
    private String firmPerson;

    /**
     * 客户联系电话
     */
    @NotBlank(message = "客户联系电话不能为空")
    @Size(max = 11, message = "客户联系电话11位")
    @Size(min = 11, message = "客户联系电话11位")
    @Pattern(regexp = "1[0-9][0-9]\\d{8}", message = "手机号格式有误")
    private String firmPhone;

    /**
     * 客户邮箱
     */
    @Email(message = "邮箱格式有误")
    private String firmEmail;

    /**
     * 客户图标 base64
     */
    private String appIcon;

    /**
     * 代理商ID
     */
    private String agentClientId;

    /**
     * 代理商销售ID
     */
    private String agentSaleId;

    /**
     * 客户状态 0待审核 1审核通过 2审核拒绝 4删除
     */
    private String firmStatus;

    /**
     * 医生端产品形态 支持app
     */
    private Boolean channelApp;

    /**
     * 医生端产品形态 支持sdk
     */
    private Boolean channelSdk;

    /**
     * 统一社会信用代码
     */
    private String firmOrgCode;
}