package com.trust.ywx.customer.dao;

import com.trust.ywx.customer.commons.UniqueIdUtils;
import com.trust.ywx.customer.dao.mapper.FirmInfoExtendMapper;
import com.trust.ywx.customer.dao.po.FirmInfoExtend;
import com.trust.ywx.customer.dao.po.FirmInfoExtendExample;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <pre></pre>
 *
 * @文件名称： FirmExtendDao.java
 * @包 路  径： com.trust.ywx.customer.dao.mapper
 * @Copyright：北京数字医信责任有限公司 (C) 2022 *
 * @Description:
 * @Version: V1.0
 * @Author： hanzhenchao
 * @Date：2022/6/16 17:31
 * @Modify：
 */
@Slf4j
@Repository
public class FirmExtendDao {
    @Resource
    FirmInfoExtendMapper extendMapper;

    public int batchInsert(String clientId, Map<String, String> propMap) {
        // 先删除
        FirmInfoExtendExample example = new FirmInfoExtendExample();
        example.createCriteria().andClientIdEqualTo(clientId)
                //根据传入的配置精准删除
                .andPropNameIn(propMap.keySet().stream().collect(Collectors.toList()));
        extendMapper.deleteByExample(example);
        // 批量添加
        List<FirmInfoExtend> list = new ArrayList<>(propMap.size());
        for (Map.Entry<String, String> prop : propMap.entrySet()) {
            if (null == prop.getValue() || "null".equalsIgnoreCase(prop.getValue())) {
                continue;
            }
            FirmInfoExtend po = new FirmInfoExtend();
            // TODO 以后要废弃该字段，新主键为int类型，数据库递增。
            po.setUniqueid(UniqueIdUtils.genrateUniqueId());
            po.setClientId(clientId);
            po.setPropName(prop.getKey());
            po.setPropValue(prop.getValue());
            list.add(po);
        }
        return extendMapper.batchInsert(list);
    }

    /**
     * 根据属性集合获取客户id
     *
     * @param propName
     * @param clientIdList
     */
    public List<String> getByPropsAndClientId(String propName, String propValue, List<String> clientIdList) {
        FirmInfoExtendExample example = new FirmInfoExtendExample();
        FirmInfoExtendExample.Criteria where = example.createCriteria();
        where.andPropNameEqualTo(propName).andPropValueEqualTo(propValue);
        if (!CollectionUtils.isEmpty(clientIdList)) {
            where.andClientIdIn(clientIdList);
        }
        List<FirmInfoExtend> poList = extendMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(poList)) {
            return Arrays.asList();
        }

        return poList.stream().map(extend -> {
            return extend.getClientId();
        }).collect(Collectors.toList());
    }

    public List<FirmInfoExtend> getListByClientId(String clientId) {
        FirmInfoExtendExample example = new FirmInfoExtendExample();
        FirmInfoExtendExample.Criteria where = example.createCriteria();
        where.andClientIdEqualTo(clientId);
        List<FirmInfoExtend> poList = extendMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(poList)) {
            return Arrays.asList();
        }

        return poList;
    }

}
