package com.trust.ywx.customer.dao;

import com.trust.ywx.customer.common.enu.FirmStatusEnum;
import com.trust.ywx.customer.common.enu.ProjectStatusEnum;
import com.trust.ywx.customer.dao.mapper.FirmInfoMapper;
import com.trust.ywx.customer.dao.po.FirmInfo;
import com.trust.ywx.customer.dao.po.FirmInfoExample;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.List;

/**
 * <pre></pre>
 *
 * @文件名称： FirmDao.java
 * @包 路  径： com.trust.ywx.customer.dao
 * @Copyright：北京数字医信责任有限公司 (C) 2022 *
 * @Description: 客户查询DAO
 * @Version: V1.0
 * @Author： hanzhenchao
 * @Date：2022/6/16 10:31
 * @Modify：
 */
@Repository
public class TrustFirmDao {

    @Resource
    private FirmInfoMapper firmInfoMapper;

    public List<FirmInfo> getListByOrgCode(String orgCode){
        FirmInfoExample example = new FirmInfoExample();
        example.createCriteria().andClientOrgCodeEqualTo(orgCode);
        return firmInfoMapper.selectByExample(example);
    }
    public List<FirmInfo> getListByFirmName(String firmName){
        FirmInfoExample example = new FirmInfoExample();
        example.createCriteria().andFirmNameEqualTo(firmName);
        return firmInfoMapper.selectByExample(example);
    }

    public List<FirmInfo> getListByIds(List<String> clientIdList, int pageSize, int pageNo) {
        FirmInfoExample example = new FirmInfoExample();
        example.createCriteria().andClientIdIn(clientIdList);
        example.page(pageNo, pageSize);
        return firmInfoMapper.selectByExample(example);
    }

    /**
     * 根据客户Id查询客户信息
     * 已删除的客户不对外暴露
     * @param clientId
     * @return
     */
    public FirmInfo getFrimByClientId(String clientId) {
        FirmInfoExample example = new FirmInfoExample();
        example.createCriteria().andClientIdEqualTo(clientId);
        return firmInfoMapper.selectOneByExample(example);
    }

    /**
     * 根据客户Id查询客户信息
     *  已删除的客户不对外暴露
     * @param clientId
     * @return
     */
    public FirmInfo getFirmBLOBsByClientId(String clientId) {
        FirmInfoExample example = new FirmInfoExample();
        example.createCriteria().andClientIdEqualTo(clientId);
        return firmInfoMapper.selectOneByExampleWithBLOBs(example);
    }


    public int insert(FirmInfo firmInfo) {
        firmInfo.setCreateTime(null);
        firmInfo.setUpdateTime(null);
        return firmInfoMapper.insertSelective(firmInfo);
    }
    public int update(FirmInfo firmInfo) {
        firmInfo.setUpdateTime(null);
        FirmInfoExample example = new FirmInfoExample();
        example.createCriteria().andClientIdEqualTo(firmInfo.getClientId());
        return firmInfoMapper.updateByExampleSelective(firmInfo,example);
    }

    /**
     * 根据客户id删除客户
     * @param clientId
     * @return
     */
    public int deleteByClientId(String clientId) {
        FirmInfo firmInfo = new FirmInfo();
        firmInfo.setUpdateTime(null);
        firmInfo.setStatus(FirmStatusEnum.FIRM_SUB_DELETE.getCode());
        FirmInfoExample example = new FirmInfoExample();
        example.createCriteria().andClientIdEqualTo(clientId);
        return firmInfoMapper.updateByExampleSelective(firmInfo,example);
    }
}
