package com.trust.ywx.customer.service.impl;

import com.fiftyonetrust.common.model.result.Page;
import com.fiftyonetrust.common.model.result.PageList;
import com.trust.ywx.customer.common.enu.FirmProperEnum;
import com.trust.ywx.customer.common.enu.NetworkTypeEnum;
import com.trust.ywx.customer.dao.DoctorProductTypeDao;
import com.trust.ywx.customer.dao.TrustFirmDao;
import com.trust.ywx.customer.dao.FirmExtendDao;
import com.trust.ywx.customer.dao.FirmGroupDao;
import com.trust.ywx.customer.dao.po.FirmInfo;
import com.trust.ywx.customer.dao.po.FirmInfoExtend;
import com.trust.ywx.customer.dao.po.FirmUsercertChannel;
import com.trust.ywx.customer.common.model.manager.response.TrustFirmDto;
import com.trust.ywx.customer.common.model.query.request.SearchCrmFirmParam;
import com.trust.ywx.customer.common.model.query.request.SearchFirmParam;
import com.trust.ywx.customer.common.model.query.request.SearchTrustFirmParam;
import com.trust.ywx.customer.common.model.manager.response.TempFirmDto;
import com.trust.ywx.customer.common.model.query.response.CrmFirmDto;
import com.trust.ywx.customer.service.query.FirmQueryService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * <pre></pre>
 *
 * @文件名称： FirmQueryServiceImpl.java
 * @包 路  径： com.trust.ywx.customer.service.impl
 * @Copyright：北京数字医信责任有限公司 (C) 2022 *
 * @Description:
 * @Version: V1.0
 * @Author： hanzhenchao
 * @Date：2022/6/15 18:43
 * @Modify：
 */
@Service
public class FirmQueryServiceImpl implements FirmQueryService {
    @Autowired
    private TrustFirmDao trustFirmDao;
    @Autowired
    private FirmGroupDao firmGroupDao;
    @Autowired
    private FirmExtendDao firmExtendDao;
    @Autowired
    private DoctorProductTypeDao doctorProductTypeDao;

    @Override
    public List<TrustFirmDto> getTrustFirmList(SearchTrustFirmParam param) {
        return doSearch(param,false);
    }

    @Override
    public PageList<List<TrustFirmDto>> getTrustFirmListWithPage(SearchTrustFirmParam param) {
        List<TrustFirmDto> dtoList = doSearch(param,true);

        // fixme 组装好麻烦，需要改进下
        PageList pageList = new PageList();
        pageList.setItems(dtoList);
        Page page = new Page();
        page.setPageNo(param.getPageNo());
        page.setPageSize(param.getPageSize());
        page.setTotal(dtoList.size());
        pageList.setPage(page);
        return pageList;
    }


    private TrustFirmDto converPoToDto(FirmInfo po) {
        if (null == po) {
            return null;
        }
        TrustFirmDto dto = new TrustFirmDto();
        BeanUtils.copyProperties(po, dto);

        // TODO 差异属性转换
        return dto;
    }

    @Override
    public TrustFirmDto getTrustFirm(String clientId) {
        FirmInfo po = trustFirmDao.getFrimByClientId(clientId);
        return converPoToDto(po);
    }

    @Override
    public TrustFirmDto getTrustFirmWithBLOBs(String clientId) {
        FirmInfo po = trustFirmDao.getFrimByClientId(clientId);
        return converPoToDto(po);
    }

    @Override
    public Map<String, String> getAllProp(String clientId) {
        if (StringUtils.isEmpty(clientId)) {
            return new HashMap<>();
        }
        Map<String, String> map = new HashMap<>(64);
        // 1、从firm_info_extend获取全量属性
        List<FirmInfoExtend> extendList = firmExtendDao.getListByClientId(clientId);
        for (FirmInfoExtend extendPo : extendList) {
            map.put(extendPo.getPropName(), extendPo.getPropValue());
        }

        //2、从firm_usercert_channel 获取医师产品形态属性 fixme 以后要去掉
        List<FirmUsercertChannel> doctorProductList = doctorProductTypeDao.getByClientId(clientId);

        for (FirmUsercertChannel doctorPo : doctorProductList) {
            if(doctorPo.getChannelType()==0){
                map.put(FirmProperEnum.DOCTOR_PRODUCT_APP.getProperName(), doctorPo.getChannelType() + "");
            }
            if(doctorPo.getChannelType()==1){
                map.put(FirmProperEnum.DOCTOR_PRODUCT_SDK.getProperName(), doctorPo.getChannelType() + "");
            }
        }
        return map;
    }

    @Override
    public List<TrustFirmDto> getFirmWhite(String firmName) {
        return null;
    }

    @Override
    public List<TempFirmDto> getTempFirmList(SearchFirmParam param) {
        return null;
    }

    @Override
    public List<CrmFirmDto> getCrmFirmList(SearchCrmFirmParam param) {
        return null;
    }

    private List<String> getClientIdListByProp(String propName, Integer propValue, List<String> clientIdList) {
        String convertPropValue = "";
        if (Objects.equals(propName, FirmProperEnum.DOCTOR_PRODUCT_SDK.getProperName())
                ||Objects.equals(propName, FirmProperEnum.DOCTOR_PRODUCT_APP.getProperName())) {
            //fixme，目前医师产品形态单独存储在firm_usercert_channel表，后期挪到firm_info_extends表
            return doctorProductTypeDao.getClientIdByProp(clientIdList, propValue);
        } else {
            // 其他属性都在firm_info_extends表
            if (Objects.equals(propName, FirmProperEnum.NETWORK_TYPE.getProperName())) {
                convertPropValue = NetworkTypeEnum.getNameOfCode(propValue);
            } else {
                convertPropValue = String.valueOf(propValue);
            }
            return firmExtendDao.getByPropsAndClientId(propName, convertPropValue, clientIdList);
        }
    }

    private List<TrustFirmDto> doSearch(SearchTrustFirmParam param , boolean isPage) {
        List<String> clientIdList = new ArrayList<>();

        // clientId筛选
        if (StringUtils.isNotEmpty(param.getClientId())) {
            FirmInfo firmInfo = trustFirmDao.getFrimByClientId(param.getClientId());
            if (null == firmInfo) {
                return Arrays.asList();
            }
            // 增加项目状态的判断
            if (null != param.getProjectStatus() && !Objects.equals(firmInfo.getProjectStatus(), param.getProjectStatus())) {
                return Arrays.asList();
            }
            // 增加实施人员的判断
            if(null!=param.getTechId() && !Objects.equals(firmInfo.getTechPersonId(),param.getTechId())){
                return Arrays.asList();
            }
            // 增加销售人员的判断
            if(null!=param.getSaleId() && !Objects.equals(firmInfo.getSalePersonId(),param.getTechId())){
                return Arrays.asList();
            }

            clientIdList.add(param.getClientId());
        }

        // 客户组筛选
        if (StringUtils.isNotEmpty(param.getGroupId())) {
            List<String> groupClientIdList = firmGroupDao.getClientIdByGroupId(param.getGroupId());
            if (CollectionUtils.isEmpty(clientIdList)) {
                clientIdList = groupClientIdList;
            } else {
                clientIdList.retainAll(groupClientIdList);
                if (CollectionUtils.isEmpty(clientIdList)) {
                    return Arrays.asList();
                }
            }
        }

        // 医师产品形态筛选 fixme 下面的查询可以合成一个
        if (null != param.getDoctorProductType()) {
            List<String> subClientIdList;
            if (StringUtils.isNotEmpty(param.getClientId())) {
                subClientIdList = getClientIdListByProp(FirmProperEnum.DOCTOR_PRODUCT_APP.getProperName(), param.getDoctorProductType(), Arrays.asList(param.getClientId()));
            } else {
                subClientIdList = getClientIdListByProp(FirmProperEnum.DOCTOR_PRODUCT_APP.getProperName(), param.getDoctorProductType(), null);
            }
            // 与clientIdList求并集
            clientIdList.retainAll(subClientIdList);
            if (CollectionUtils.isEmpty(clientIdList)) {
                return Arrays.asList();
            }

            if (StringUtils.isNotEmpty(param.getClientId())) {
                subClientIdList = getClientIdListByProp(FirmProperEnum.DOCTOR_PRODUCT_SDK.getProperName(), param.getDoctorProductType(), Arrays.asList(param.getClientId()));
            } else {
                subClientIdList = getClientIdListByProp(FirmProperEnum.DOCTOR_PRODUCT_SDK.getProperName(), param.getDoctorProductType(), null);
            }
            // 与clientIdList求并集
            clientIdList.retainAll(subClientIdList);
            if (CollectionUtils.isEmpty(clientIdList)) {
                return Arrays.asList();
            }
        }

        /**
         * 属性过滤暂时没想到好办法，只能一个个去查询了
         */
        if (null != param.getNetworkType()) {
            List<String> subClientIdList = getClientIdListByProp(FirmProperEnum.NETWORK_TYPE.getProperName(), param.getNetworkType(), clientIdList);
            // 与clientIdList求并集
            clientIdList.retainAll(subClientIdList);
            if (CollectionUtils.isEmpty(clientIdList)) {
                return Arrays.asList();
            }
        }
        if (null != param.getChargingMode()) {
            List<String> subClientIdList = getClientIdListByProp(FirmProperEnum.CHARGING_MODE.getProperName(), param.getChargingMode(), clientIdList);
            // 与clientIdList求并集
            clientIdList.retainAll(subClientIdList);
            if (CollectionUtils.isEmpty(clientIdList)) {
                return Arrays.asList();
            }
        }
        if (null != param.getPublicProductType()) {
            List<String> subClientIdList = getClientIdListByProp(FirmProperEnum.PUBLIC_PRODUCT_TYPE.getProperName(), param.getPublicProductType(), clientIdList);
            // 与clientIdList求并集
            clientIdList.retainAll(subClientIdList);
            if (CollectionUtils.isEmpty(clientIdList)) {
                return Arrays.asList();
            }
        }
        List<FirmInfo> firmList = null;
        if (isPage) {
            firmList = trustFirmDao.getListByIds(clientIdList, 100000, 0);
        } else {
            if (clientIdList.size() < param.getPageSize() * param.getPageNo()) {
                return Arrays.asList();
            }
            firmList = trustFirmDao.getListByIds(clientIdList, param.getPageSize(), param.getPageNo());
        }

        List<TrustFirmDto> dtoList = new ArrayList<>(firmList.size());
        if (CollectionUtils.isEmpty(firmList)) {
            return Arrays.asList();
        }
        for (FirmInfo po : firmList) {
            TrustFirmDto dto = converPoToDto(po);
            dtoList.add(dto);
        }
        return dtoList;
    }
}
