package com.trust.ywx.customer.service.manager.impl;

import com.trust.ywx.customer.common.enu.*;
import com.trust.ywx.customer.commons.UniqueIdUtils;
import com.trust.ywx.customer.dao.TempFirmDao;
import com.trust.ywx.customer.dao.po.FirmInfoTemp;
import com.trust.ywx.customer.common.model.manager.request.TempFirmParam;
import com.trust.ywx.customer.common.model.manager.request.TrustFirmParam;
import com.trust.ywx.customer.common.exception.BizException;
import com.trust.ywx.customer.service.manager.FirmManagerService;
import com.trust.ywx.customer.service.manager.TempFirmManagerService;
import lombok.SneakyThrows;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <pre></pre>
 *
 * @文件名称： FirmManagerServiceImpl.java
 * @包 路  径： com.trust.ywx.customer.service.manager.impl
 * @Copyright：北京数字医信责任有限公司 (C) 2022 *
 * @Description: 客户管理实现类
 * @Version: V1.0
 * @Author： hanzhenchao
 * @Date：2022/6/20 12:50
 * @Modify：
 */
@Service
public class TempFirmManagerServiceImpl implements TempFirmManagerService {
    @Resource
    TransactionTemplate transactionTemplate;
    @Resource
    TempFirmDao tempFirmDao;
    @Resource
    FirmManagerService firmManagerService;

    @Override
    public boolean add(TempFirmParam param) throws BizException {
        // 参数校验
        //checkParam(param);
        // 业务校验
        checkBiz(param);
        // 入库
        return doAddTemp(param);
    }
    @Override
    public boolean update(TempFirmParam param) throws BizException {
        // 参数校验
        //checkParam(param);
        // 业务校验
        checkBiz(param);
        // 入库
        return doUpdateTemp(param);
    }

    @Override
    public boolean audit(String clientId) throws BizException {
        // 校验clientId
        FirmInfoTemp tempFirm = tempFirmDao.getByClientId(clientId);
        if (null == tempFirm) {
            throw new BizException(FirmErrEum.FIRM_NOT_EXIST);
        }

        // 封装可信客户参数
        TrustFirmParam trustFirmParam = buildTrustFirmParam(tempFirm);

        return doAudit(trustFirmParam, tempFirm);
    }


    @Override
    public boolean reject(String clientId) throws BizException {
        // 业务校验
        FirmInfoTemp tempFirm = tempFirmDao.getByClientId(clientId);
        if (null == tempFirm) {
            throw new BizException(FirmErrEum.CLIENTID_CHECK_ERR);
        }

        int num = tempFirmDao.auditReject(clientId);
        return num == 1;
    }

    @Override
    public boolean delete(String clientId) {
        // fixme 此处应该为物理删除，否则客户在此添加时 客户名称就重复了
        return true;
    }

    private boolean doUpdateTemp(TempFirmParam param) {
        FirmInfoTemp temp = convertToTempFirm(param, false);
        return tempFirmDao.update(temp)==1;
    }

    private void checkBiz(TempFirmParam param) throws BizException {
        // firmName冲突校验
        List<FirmInfoTemp> list = tempFirmDao.getListByFirmName(param.getFirmName());
        if (null != list && list.size() > 0) {
            throw new BizException(FirmErrEum.FIRM_NAME_CHECK_ERR);
        }
    }

    private boolean doAddTemp(TempFirmParam param) {
        FirmInfoTemp temp = convertToTempFirm(param, true);
        tempFirmDao.insert(temp);
        return true;

    }

    private FirmInfoTemp convertToTempFirm(TempFirmParam param, boolean isAdd) {
        FirmInfoTemp temp = new FirmInfoTemp();
        BeanUtils.copyProperties(param, temp);
        if (isAdd) {
            temp.setUniqueid(UniqueIdUtils.genrateUniqueId());
            temp.setClientId(UniqueIdUtils.generateClientId());
        }else{
            temp.setClientId(param.getClientId());
        }
        temp.setFirmOrgCode(param.getFirmOrgCode());
        temp.setFirmStatus(TempFirmStatusEnum.TEMP_UNAUTHEN.getCode() + "");
        temp.setJuridicalName(param.getFirmPerson());
        if (param.getChannelApp() != null && true == param.getChannelApp()
                && param.getChannelSdk() != null && true == param.getChannelSdk()) {
            temp.setChannel("2");
        } else if (param.getChannelSdk() != null && true == param.getChannelSdk()) {
            temp.setChannel("1"); //sdk
        } else if (param.getChannelApp() != null && false == param.getChannelApp()
                && param.getChannelSdk() != null && false == param.getChannelSdk()) {
            temp.setChannel(null);
        } else {
            temp.setChannel("0"); //app不能默认app，可能不选channel
        }
        return temp;

    }

    private boolean doAudit(TrustFirmParam trustFirmParam, FirmInfoTemp temp) {
        transactionTemplate.execute(new TransactionCallbackWithoutResult() {
            @SneakyThrows
            @Override
            protected void doInTransactionWithoutResult(TransactionStatus transactionStatus) {
                // 1、添加可信客户
                firmManagerService.add(trustFirmParam);
                // 2、更新临时客户状态为已审核
                int num = tempFirmDao.auditPass(temp.getClientId());
                if (num != 1) {
                    throw new BizException(FirmErrEum.FIRM_SYN_ERR);
                }
            }
        });
        return true;
    }

    private TrustFirmParam buildTrustFirmParam(FirmInfoTemp temp) {
        TrustFirmParam trustFirmParam = new TrustFirmParam();
        trustFirmParam.setFirmPerson(temp.getFirmPerson()); //厂商联系人 （不是法人）
        trustFirmParam.setFirmName(temp.getFirmName());
        trustFirmParam.setFirmEmail(temp.getFirmEmail());
        String phone = StringUtils.isBlank(temp.getJuridicalPhone()) ? temp.getFirmPhone() :
                temp.getJuridicalPhone();
        trustFirmParam.setFirmPhone(phone);
        trustFirmParam.setFirmAddress(temp.getFirmAddress());
        trustFirmParam.setClientOrgCode(temp.getFirmOrgCode());
        trustFirmParam.setAgentSaleId(temp.getAgentSaleId());
        trustFirmParam.setAppIcon(temp.getFirmLogo());
        // 医师端产品形态
        List<String> doctorProTypeList = new ArrayList<>();
        if ("1".equals(temp.getChannel())) {
            doctorProTypeList.add(DoctorProductTypeEnum.SDK.getName());
        } else if ("2".equals(temp.getChannel())) {
            doctorProTypeList.add(DoctorProductTypeEnum.APP.getName());
            doctorProTypeList.add(DoctorProductTypeEnum.SDK.getName());
        } else {
            doctorProTypeList.add(DoctorProductTypeEnum.APP.getName());

        }

        trustFirmParam.setDoctorProductType(doctorProTypeList);
        trustFirmParam.setCertAlg("RSA");
        trustFirmParam.setAppSecret(UniqueIdUtils.generateAppSecret());
        // TODO 什么时候审核临时客户会携带clientId？？
        String clientId = null;
        if (StringUtils.isBlank(temp.getClientId())) {
            clientId = UniqueIdUtils.generateClientId();
        } else {
            clientId = temp.getClientId();
        }
        trustFirmParam.setAgentClientId(temp.getAgentClientId());
        trustFirmParam.setClientId(clientId);
        trustFirmParam.setCreateTime(new Date());
        // 手动制章默认
        trustFirmParam.setUserStampType(CommonStatusEnum.YES.getCode());
        // 信步云默认
        trustFirmParam.setCertAgency("1");
        trustFirmParam.setFirmStatus(FirmStatusEnum.FIRM_SUB_NORMAL.getCode());
        trustFirmParam.setSmsSend(CommonStatusEnum.YES.getCode());
        String channel = temp.getChannel();
        if ("1".equals(channel) || "2".equals(channel)) {
            //TODO 引入nacos
//            String msspAppid = config.getString("ywq.core.common.ywq.mssp.appid", "");
//            trustFirmParam.setMsspAppid(msspAppid);
        }

        // 客户类型：互联网医院
        trustFirmParam.setNetworkType(NetworkTypeEnum.IT.getCode());
        // app设置图章：可以
        trustFirmParam.setUserStampIsAppset(CommonStatusEnum.YES.getCode());
        // 客户图章类型：圆章
        trustFirmParam.setFirmStampType(FirmStampEnum.ROUND.getCode());

        return trustFirmParam;
    }

}
